'use strict';

var _ = require('lodash');
var q = require('q');
var Task = require('./Task');

function Target(name, dependencies, description) {
    this.name = name || 'unnamed';
    this.dependencies = dependencies || [];
    this.description = description || '';
    this._tasks = [];
}
module.exports = Target;

Target.prototype.task = function (fns) {

    this._tasks.push(new Task(fns));
    return this;
};

Target.prototype.run = function (reporter) {
    var target = this;

    return q()
        .then(function () {
            if (reporter && _.isFunction(reporter.beforeTarget)) {
                reporter.beforeTarget(target);
            }
        })
        .then(function () {
            var promise = q();

            _.each(target._tasks, function (task) {
                promise = promise.then(function () {
                    return task.run();
                });
            });

            return promise;
        })
        .then(function () {
            if (reporter && _.isFunction(reporter.afterTarget)) {
                reporter.afterTarget();
            }
        });
};
